#pragma once
#include <typeinfo>
#include "cmMath.h"

class BoxCollision;
class SphereCollision;
class MeshCollision;
class Entity;

#define DECLARE_EXTERNAL_COLLISION_DETECTION_FUNCTION(_lhs, _rhs) \
	static bool DoesCollide(_lhs* p##_lhs, _rhs* p##_rhs) { return DoesCollide(p##_rhs, p##_lhs); }	\
	static bool DoesCollide(_rhs* p##_rhs, _lhs* p##_lhs);	\

#define IMPLEMENT_EXTERNAL_COLLISION_DETECTION_FUNCTION(_lhs, _rhs) \
	bool CollisionType::DoesCollide(_rhs* p##_rhs, _lhs* p##_lhs)

#define DECLARE_INTERNAL_COLLISION_DETECTION_FUNCTION(_type)				\
	static bool DoesCollide(_type* plhs, _type* prhs);	\

#define IMPLEMENT_INTERNAL_COLLISION_DETECTION_FUNCTION(_type)\
	bool CollisionType::DoesCollide(_type* plhs, _type* prhs)	\

#define COLLISIONTEST(type)									\
{															\
type* pType = dynamic_cast<type*>(pOther);		\
if (pType)													\
{															\
return CollisionType::DoesCollide(this, pType);				\
}															\
}															\

#define IMPLEMENT_COLLISIONDETECTION()						\
	if (!bEnabled) return false;							\
	COLLISIONTEST(BoxCollision);							\
	COLLISIONTEST(SphereCollision);							\
	COLLISIONTEST(MeshCollision);							\
	return Fail();											\


class CollisionType
{
public:
	CollisionType(Entity* pEntity = nullptr)
		: m_pEntity(pEntity)
	{
	}
	virtual ~CollisionType() {}

	virtual bool DoesCollide(CollisionType* pOther) = 0;
	bool Fail() { return false; }
	
	DECLARE_EXTERNAL_COLLISION_DETECTION_FUNCTION(BoxCollision, SphereCollision)
	DECLARE_EXTERNAL_COLLISION_DETECTION_FUNCTION(BoxCollision, MeshCollision)
	DECLARE_EXTERNAL_COLLISION_DETECTION_FUNCTION(SphereCollision, MeshCollision)

	DECLARE_INTERNAL_COLLISION_DETECTION_FUNCTION(BoxCollision)
	DECLARE_INTERNAL_COLLISION_DETECTION_FUNCTION(SphereCollision)
	DECLARE_INTERNAL_COLLISION_DETECTION_FUNCTION(MeshCollision)

	Entity* m_pEntity;
	bool bEnabled = true;
};


class BoxCollision : public CollisionType
{
public:
	virtual bool DoesCollide(CollisionType* pOther);
	// Whatever vars
};

class SphereCollision : public CollisionType
{
public:
	SphereCollision(Entity* pEntity, float fRadius)
		: CollisionType(pEntity)
		, m_fRadius(fRadius)
	{}
	virtual bool DoesCollide(CollisionType* pOther);
	
	float m_fRadius = 0.0f;
};

class MeshCollision	: public CollisionType
{
public:
	MeshCollision(Entity* pEntity, cmMeshCollider meshCollider)
		: CollisionType(pEntity)
		, m_meshCollider(meshCollider)
	{}
	virtual bool DoesCollide(CollisionType* pOther);
	
	cmMeshCollider m_meshCollider;

};

class NoCollision : public CollisionType
{
	virtual bool DoesCollide(CollisionType* pOther) { return false; }
};


