#include "EntityManager.h"
#include "cmPlatform.h"


const float EntityManager::ms_fRestMagnitudeThreshold = 0.001f;

EntityManager::EntityManager()
{}

EntityManager::~EntityManager()
{}

void EntityManager::Update()
{
	ProcessRemovedEntities();

	m_bSystemAtRest = true;
	for (EntityMap::iterator iter = m_entities.begin(); iter != m_entities.end(); ++iter)
	{
		iter->second->PreUpdate();
	}

	for (EntityMap::iterator outer = m_entities.begin(); outer != m_entities.end(); ++outer)
	{
		std::shared_ptr<Entity> spOuter = outer->second;

		// Collisions
		for (EntityMap::iterator inner = outer; inner != m_entities.end(); ++inner)
		{
			// Hold them in shared pointers to keep them valid through their ordeal
			std::shared_ptr<Entity> spInner = inner->second;
			if (inner != outer && DetectCollision(spInner.get(), spOuter.get()))
			{
				spInner->OnCollision(spOuter.get());
				spOuter->OnCollision(spInner.get());
			}
		}
	}

	for (EntityMap::iterator iter = m_entities.begin(); iter != m_entities.end(); ++iter)
	{
		iter->second->Update();

		m_bSystemAtRest &= iter->second->m_velocity.magnitude() < ms_fRestMagnitudeThreshold;
	}
}

bool EntityManager::DetectCollision(Entity* pLHS, Entity* pRHS)
{
	return pLHS->m_collision && pRHS->m_collision
		&& pLHS->m_collision->DoesCollide(pRHS->m_collision);
}

void EntityManager::Render()
{
	for (const auto& pair : m_entities)
		pair.second->Render();
}

void EntityManager::AddEntity(std::shared_ptr<Entity> entity)
{
	m_entities[entity->m_nId] = entity;
}

void EntityManager::RemoveEntity(Entity* pEntity)
{
	if (pEntity)
	{
		RemoveEntity(pEntity->m_nId);
	}
}

void EntityManager::RemoveEntity(int nId)
{
	m_removalQueue.insert(nId);
}

void EntityManager::ProcessRemovedEntities()
{
	for (const auto& id : m_removalQueue)
	{
		m_entities.erase(id);
		// TODO: Fire an event of some kind?
	}
	m_removalQueue.clear();
}