#pragma once

#include "Entity.h"
#include <map>
#include <set>
#include <memory>
#include <vector>

class EntityManager
{
public:

	static const float ms_fRestMagnitudeThreshold;

	EntityManager();
	virtual ~EntityManager();

	// Management
	void Update();
	bool DetectCollision(Entity* pLHS, Entity* pRHS);
	void Render();
	void Purge() { m_entities.clear(); }
	// Queries
	template<typename T>
	void GetEntites(std::vector<T*>& entities);

	// Entity Lifecycle
	template <typename T, typename ...Args>
	T* SpawnEntity(Args...);
	void AddEntity(std::shared_ptr<Entity> entity);

	void RemoveEntity(Entity* pEntity);
	void RemoveEntity(int nId);
	void ProcessRemovedEntities();

	int GetNextEntityId() { return ++m_nLastEntityId; }
	int m_nLastEntityId = 0;

	typedef std::map<int, std::shared_ptr<Entity> > EntityMap;
	EntityMap m_entities;

	bool IsSystemAtRest() { return m_bSystemAtRest; }
	bool m_bSystemAtRest = true;

	std::set<int> m_removalQueue;
};

#include "EntityManager.inl"

