#include "FlameHazard.h"
#include "GGJ2015.h"
#include "Spaceman.h"

const float FlameHazard::fMinFlameFrequencyMS = 2000.0f; // minimum frequency between flame bursts
const float FlameHazard::fMaxFlameFrequencyMS = 5000.0f; // maximum frequency between flame bursts
const float FlameHazard::fMinFlameBurstTimeMS = 1000.0f; // minimum amount of time a flame can burst for
const float FlameHazard::fMaxFlameBurstTimeMS = 2000.0f; // maximum amount of time a flame can burst for
RNG FlameHazard::ms_RNG = RNG();

FlameHazard::FlameHazard(cmTransform& transform, const std::string& sTriggerName)
	: Hazard(transform)
{
	// set any assets we might need in the hazard itself
	m_collision = new MeshCollision(this, cmLocatorManager::Get().GetTriggerCollision(sTriggerName));

	// set the frequency and the burst
	float fFrequencyRange = fMaxFlameFrequencyMS - fMinFlameFrequencyMS;
	m_fFlameFrequencyMS = fMinFlameFrequencyMS + (fFrequencyRange * ms_RNG.GetFloat());

	float fBurstRange = fMaxFlameBurstTimeMS - fMinFlameBurstTimeMS;
	m_fFlameBurstMS = fMinFlameBurstTimeMS + (fBurstRange * ms_RNG.GetFloat());
	m_fAnimationTime = 1000.0f;
	m_fAnimationTimer = 0.0f;
}

FlameHazard::~FlameHazard()
{}

void FlameHazard::Update()
{
	m_fAnimationTimer += gPlatform->GetDeltaTime();
	if (m_fAnimationTimer > m_fAnimationTime)
	{
		m_fAnimationTimer -= m_fAnimationTime;
	}

	

	m_fFlameTimer += gPlatform->GetDeltaTime();

	// toggle between a burst of flame or a waiting period
	float fTimeout = m_bFlamesOn ? m_fFlameBurstMS : m_fFlameFrequencyMS;
	
	if (m_fFlameTimer >= fTimeout)
	{
		SetFlames(!m_bFlamesOn);
	}
}

void FlameHazard::Render()
{
	Hazard::Render();

	if (m_bFlamesOn)
	{
		// Render

		
		gRenderer->MatrixPush();
		gRenderer->SetProjectionMode(Mode3D);
		gRenderer->MatrixMultiply(m_transform.matrix());

		gRenderer->MatrixPush();
		int index = (int)(m_fAnimationTimer / m_fAnimationTime * 11);
		wchar_t buffer[64];
		swprintf(buffer, sizeof(buffer) / sizeof(wchar_t), L"fire_1_%03d.tga", index);

		gRenderer->BindTexture(gRenderer->GetTexture(buffer).nId);
		cmAlphaShader::Get().Activate();
		gRenderer->SetBlendMode(ModeAdd);
		gRenderer->SetColor(cmColor(1.0f, 0.5f, 0.0f, 1.0f));
		GGJ2015::Get()->GetFlameMesh().Render();
		gRenderer->MatrixPop();
		gRenderer->MatrixPop();
		gRenderer->SetBlendMode(ModeNormal);
	}
}

void FlameHazard::OnCollision(Entity* pEntity)
{
	if (m_bFlamesOn && pEntity)
	{
		Spaceman* pSpaceman = dynamic_cast<Spaceman*>(pEntity);
		if (pSpaceman)
		{
			// Remove him from play
			GGJ2015::Get()->m_pEntityManager->RemoveEntity(pEntity);

			// TODO: Spawn some gib, play some sound or something
		}
	}
}

void FlameHazard::SetFlames(bool bFlamesOn)
{
	if (m_bFlamesOn != bFlamesOn)
	{
		m_fFlameTimer = 0.0f;
		m_bFlamesOn = bFlamesOn;
	}
}
