#include "GGJ2015.h"
#include "cmPlatform.h"

// Entities
#include "Spaceman.h"

// Screens
#include "MainMenuScreen.h"
#include "SplashScreen.h"
#include "TutorialScreen.h"
#include "InGameScreen.h"
#include "CreditsScreen.h"

#include "CollisionType.h"
#include <typeinfo>

GGJ2015* GGJ2015::Get()
{
	return (GGJ2015*)(gGame);
}

void GGJ2015::Init()
{
	/*
	BoxCollision box;
	SphereCollision sphere;
	MeshCollision mesh;
	//*/
	
	gRenderer->EnableBlend(true);
	gRenderer->EnableTextures(true);
// 	gRenderer->EnableDepthMask(false);
// 	gRenderer->EnableDepthTest(false);
// 	gRenderer->EnableDepthMask(true);
// 	gRenderer->EnableDepthTest(true);
	cmFont::InitAtlases();

	LoadStrings(L"strings\\strings_en.txt");
	LoadStrings(L"strings\\sound.txt");
	

	m_flameMesh = gRenderer->LoadBinaryMesh(L"meshes\\hazard_mesh.cmbm");


	

	LoadRoom(1);
	
	m_sceneCollision = gRenderer->LoadBinaryMesh(L"meshes\\visual_001_collision_001.cmbm");
	m_sphere = gRenderer->LoadBinaryMesh(L"meshes\\sphere.cmbm");

	m_dudeIdleMeshFace = gRenderer->LoadBinaryMesh(L"meshes\\spaceman_01_idle_face.cmbm");
	m_dudeIdleMeshPhong = gRenderer->LoadBinaryMesh(L"meshes\\spaceman_01_idle_gloss.cmbm");
	m_dudeIdleMeshMatte = gRenderer->LoadBinaryMesh(L"meshes\\spaceman_01_idle_matte.cmbm");
	m_dudeIdleMeshSpecular = gRenderer->LoadBinaryMesh(L"meshes\\spaceman_01_idle_visor.cmbm");

	m_dudeKickLMeshFace = gRenderer->LoadBinaryMesh(L"meshes\\spaceman_01_kick_l_face.cmbm");
	m_dudeKickLMeshPhong = gRenderer->LoadBinaryMesh(L"meshes\\spaceman_01_kick_l_gloss.cmbm");
	m_dudeKickLMeshMatte = gRenderer->LoadBinaryMesh(L"meshes\\spaceman_01_kick_l_matte.cmbm");
	m_dudeKickLMeshSpecular = gRenderer->LoadBinaryMesh(L"meshes\\spaceman_01_kick_l_visor.cmbm");

	m_dudeKickRMeshFace = gRenderer->LoadBinaryMesh(L"meshes\\spaceman_01_kick_r_face.cmbm");
	m_dudeKickRMeshPhong = gRenderer->LoadBinaryMesh(L"meshes\\spaceman_01_kick_r_gloss.cmbm");
	m_dudeKickRMeshMatte = gRenderer->LoadBinaryMesh(L"meshes\\spaceman_01_kick_r_matte.cmbm");
	m_dudeKickRMeshSpecular = gRenderer->LoadBinaryMesh(L"meshes\\spaceman_01_kick_r_visor.cmbm");

	m_dudePunchLMeshFace = gRenderer->LoadBinaryMesh(L"meshes\\spaceman_01_punch_l_face.cmbm");
	m_dudePunchLMeshPhong = gRenderer->LoadBinaryMesh(L"meshes\\spaceman_01_punch_l_gloss.cmbm");
	m_dudePunchLMeshMatte = gRenderer->LoadBinaryMesh(L"meshes\\spaceman_01_punch_l_matte.cmbm");
	m_dudePunchLMeshSpecular = gRenderer->LoadBinaryMesh(L"meshes\\spaceman_01_punch_l_visor.cmbm");

	m_dudePunchRMeshFace = gRenderer->LoadBinaryMesh(L"meshes\\spaceman_01_punch_r_face.cmbm");
	m_dudePunchRMeshPhong = gRenderer->LoadBinaryMesh(L"meshes\\spaceman_01_punch_r_gloss.cmbm");
	m_dudePunchRMeshMatte = gRenderer->LoadBinaryMesh(L"meshes\\spaceman_01_punch_r_matte.cmbm");
	m_dudePunchRMeshSpecular = gRenderer->LoadBinaryMesh(L"meshes\\spaceman_01_punch_r_visor.cmbm");

	

	

	m_BufferA = gRenderer->GenerateFramebuffer();
	gRenderer->AttachFramebufferTextureAndDepth(m_BufferA, gPlatform->GetWindowSize()*0.25f, ModeLinear);

	m_BufferB = gRenderer->GenerateFramebuffer();
	gRenderer->AttachFramebufferTextureAndDepth(m_BufferB, gPlatform->GetWindowSize()*0.25f, ModeLinear);

	RegisterScreens();

	m_pEntityManager = new EntityManager();
	m_pLevelManager = new LevelManager();
	m_pGameStateManager = new GameStateManager();

	Reset();
}



void GGJ2015::RegisterScreens()
{
	cmScreenManager& screenManager = cmScreenManager::Get();
	screenManager.RegisterScreen(InGameScreen::ms_szScreenName, new InGameScreen());
	screenManager.RegisterScreen(MainMenuScreen::ms_szScreenName, new MainMenuScreen());
	screenManager.RegisterScreen(SplashScreen::ms_szScreenName, new SplashScreen());
	screenManager.RegisterScreen(TutorialScreen::ms_szScreenName, new TutorialScreen());
	screenManager.RegisterScreen(CreditsScreen::ms_szScreenName, new CreditsScreen());
}

void GGJ2015::Update()
{
	// Managers
	m_pEntityManager->Update();
	m_pGameStateManager->Update();

	//gRenderer->SetProjectionMode(Mode3D); //this is here so we can raycast
	//gRenderer->UpdateModelviewMatrix();
	//gRenderer->UpdateProjectionMatrix();
	//gRenderer->UpdateMouseVectors();
}

void GGJ2015::Render()
{
	// Managers
	

	m_vLightPos = Vector3(0, 0, 90);

	gRenderer->BindFramebufferId(0);

	gRenderer->Clear();
	cmFlatShader::Get().Activate();
	cmQuad2D background;
	background.SetTexture(L"background_layer_01.tga");
	background.SetSize(Vector3(1, 1));
	background.SetColor(cmColor(255.0f / 255.0f, 255.0f / 255.0f, 255.0f / 255.0f, 1.0f));
	background.SetScaleMode(ModeScaledRelativeToWidthAspect);
	background.Render();

	
	m_pGameStateManager->Render();

// 	cmQuad2D fullScreenQuad;
// 	fullScreenQuad.SetSize(Vector3(1,1));
// 	fullScreenQuad.SetScaleMode(ModeScaled);
// 	
// 	gRenderer->BindFramebuffer(m_BufferA);
// 	gRenderer->Clear();
// 	cmAlphaShader::Get().Activate();
// 	RenderBackground();
// 	
// 	RenderScene();
// 	//gRenderer->SetColorOffset(cmColor(-0.4f, -0.4f, -0.4f, 0.0f));
// 	//RenderLight();
// 	//gRenderer->SetColorOffset(cmColor(0.0f, 0.0f, 0.0f, 0.0f));
// 
// 	gRenderer->BindFramebuffer(m_BufferB);
// 	gRenderer->Clear();
// 	cmBlurHShader::Get().Activate();
// 	cmBlurHShader::Get().SetBlurAmount(1.0f);
// 	gRenderer->BindTexture(m_BufferA.nTextureId);
// 	fullScreenQuad.Render();
// 
// 	gRenderer->BindFramebuffer(m_BufferA);
// 	gRenderer->Clear();
// 	cmBlurVShader::Get().Activate();
// 	cmBlurVShader::Get().SetBlurAmount(1.0f);
// 	gRenderer->BindTexture(m_BufferB.nTextureId);
// 	fullScreenQuad.Render();
// 	
// 	gRenderer->BindFramebufferId(0);
// 	gRenderer->Clear();
// 	cmFlatShader::Get().Activate();
// 	RenderBackground();
// 	RenderScene();
// 	RenderLight();
// 
// 	cmFlatShader::Get().Activate();
// 	gRenderer->BindTexture(m_BufferA.nTextureId);
// 	fullScreenQuad.SetAlpha(1.0f);
// 	fullScreenQuad.SetBlendMode(ModeAdd);
// 	fullScreenQuad.Render();
}

void GGJ2015::RenderBackground()
{
// 	cmQuad2D background;
// 	background.SetTexture(L"TEX_WHITE");
// 	background.SetSize(Vector3(1, 1));
// 	background.SetColor(cmColor(0.0f / 255.0f, 0.0f / 255.0f, 0.0f / 255.0f, 1.0f));
// 	background.SetScaleMode(ModeScaled);
// 	background.Render();
}

void GGJ2015::RenderScene()
{
// 	gRenderer->MatrixPush();
// 	m_camera.Look();
// 	gRenderer->SetProjectionMode(Mode3D);
// 	gRenderer->UpdateModelviewMatrix();
// 	gRenderer->UpdateProjectionMatrix();
//  	gRenderer->SetBlendMode(ModeNormal);
// 	gRenderer->SetColor(cmColor(1.0f, 1.0f, 1.0f, 1.0f));
// 	//gRenderer->SetColorOffset(cmColor(0.0f, 0.0f, 0.0f, 0.4f));
// 	gRenderer->EnableDepthMask(true);
// 	gRenderer->EnableDepthTest(true);
// 	gRenderer->BindTexture(m_sceneTexture.nId);
// 	m_scene.Render();
// 	gRenderer->MatrixPop();
// 	gRenderer->SetColorOffset(cmColor(0.0f, 0.0f, 0.0f, 0.0f));
}

void GGJ2015::RenderLight()
{
	/*gRenderer->MatrixPush();
	m_camera.Look();
	gRenderer->SetProjectionMode(Mode3D);
	gRenderer->MatrixTranslate(m_vLightPos);
	cmFlatShader::Get().Activate();
	gRenderer->SetBlendMode(ModeNormal);
	gRenderer->SetColor(cmColor(1.0f, 1.0f, 0.0f, 1.0f));
	gRenderer->EnableDepthMask(true);
	gRenderer->EnableDepthTest(true);
	gRenderer->BindTexture(gRenderer->GetTexture(L"TEX_WHITE").nId);
	m_sphere.Render();
	gRenderer->MatrixPop();*/
	
// 	float fGap = 30.0f;
// 	for(int i=0;i<3;++i)
// 	{
// 		gRenderer->MatrixPush();
// 		m_camera.Look();
// 		gRenderer->SetProjectionMode(Mode3D);
// 		gRenderer->MatrixTranslate(Vector3(280 - fGap + (float)i*fGap, -75, 10));
// 		gRenderer->UpdateModelviewMatrix();
// 		gRenderer->UpdateProjectionMatrix();
// 		cmPhongShader::Get().Activate();
// 		cmPhongShader::Get().SetAmbient(0.25f);
// 		cmPhongShader::Get().SetLightPos(m_vLightPos * m_camera.GetTransform().viewMatrix());
// 		cmPhongShader::Get().SetSpecularColor(cmColor());
// 		cmPhongShader::Get().SetSpecularGloss(30.0f);
// 		cmPhongShader::Get().SetSpecularAmount(0.5f);
// 		cmPhongShader::Get().SetModelViewMatrix(gRenderer->GetModelViewMatrix());
// 		cmPhongShader::Get().SetNormalMatrix(gRenderer->GetNormalMatrix());
// 		gRenderer->SetBlendMode(ModeNormal);
// 		gRenderer->SetColor(cmColor(0.75f, 0.75f, 0.75f, 1.0f));
// 		gRenderer->EnableDepthMask(true);
// 		gRenderer->EnableDepthTest(true);
// 		gRenderer->BindTexture(gRenderer->GetTexture(L"TEX_WHITE").nId);
// 		m_sphere.Render();
// 		gRenderer->MatrixPop();
// 	}
}
cmColor GGJ2015::GetPlayerColour(Player player)
{
	static cmColor colours[] =
	{
		cmColor(0.3f, 0.3f, 0.3f, 1.0f),
		cmColor(0.85f, 0.0f, 0.1f, 1.0f),
		cmColor(0.0f, 0.3f, 0.86f, 1.0f)
	};

	return colours[(int)player];
}

bool GGJ2015::OnMouseDown(const cmMouseButtons& button, const Vector3& vPos)
{
	m_pGameStateManager->OnMouseDown(button, vPos);
	return true;
}

bool GGJ2015::OnMouseUp(const cmMouseButtons& button, const Vector3& vPos)
{
	m_pGameStateManager->OnMouseUp(button, vPos);
	return true;
}

bool GGJ2015::OnMouseClick(const cmMouseButtons& button, const Vector3& vPos)
{
	m_pGameStateManager->OnMouseClick(button, vPos);
	return true;
}

bool GGJ2015::OnTouchStart()
{
	m_pGameStateManager->OnTouchStart();
	return true;
}

bool GGJ2015::OnTouchEnd()
{
	m_pGameStateManager->OnTouchEnd();
	return true;
}

void GGJ2015::Reset()
{
	m_fZConstraint = 20.0f;
}

void GGJ2015::LoadRoom(int nRoomId)
{
	// common
	wchar_t buffer[128];
	wchar_t sRoomId[32];
	swprintf_s(sRoomId, L"%03d", nRoomId);
	
	swprintf_s(buffer, L"meshes\\room_windows_alpha.cmbm", sRoomId);
	m_sceneAlpha = gRenderer->LoadBinaryMesh(L"meshes\\room_windows_alpha.cmbm");

	swprintf_s(buffer, L"meshes\\room_%s_matte.cmbm", sRoomId);
	m_sceneMatte = gRenderer->LoadBinaryMesh(buffer);

	swprintf_s(buffer, L"meshes\\room_%s_spec.cmbm", sRoomId);
	m_sceneSpecular = gRenderer->LoadBinaryMesh(buffer);

	swprintf_s(buffer, L"meshes\\visual_%s_details_%s.cmbm", sRoomId, sRoomId);
	m_sceneDetail = gRenderer->LoadBinaryMesh(buffer);

	swprintf_s(buffer, L"Room_%s_COMP.tga", sRoomId);
	m_sceneTexture = buffer;

	swprintf_s(buffer, L"locators\\room_%s_locators.cloc", sRoomId);
	cmLocatorManager::Get().Load(buffer);

	swprintf_s(buffer, L"triggers\\room_%s_triggers.trig", sRoomId);
	cmLocatorManager::Get().LoadTriggers(buffer);

	swprintf_s(buffer, L"meshes\\room_%s_collision.cmbm", sRoomId);
	m_collision.Load(buffer);
	
}