#pragma once
#include <memory>
#include "cmGame.h"
#include "Camera.h"

#include "GameState.h"
#include "Player.h"

// Managers
#include "EntityManager.h"
#include "LevelManager.h"
#include "GameStateManager.h"
#include "Stages.h"
class Spaceman;

class GGJ2015 :
	public cmGame
{
public:

	GGJ2015(){}

	static GGJ2015* Get();

	virtual void Init();
	virtual void RegisterScreens();
	virtual void Update();
	virtual void Render();

	virtual bool OnMouseDown(const cmMouseButtons& button, const Vector3& vPos);
	virtual bool OnMouseUp(const cmMouseButtons& button, const Vector3& vPos);
	virtual bool OnMouseClick(const cmMouseButtons& button, const Vector3& vPos);
	virtual bool OnTouchStart();
	virtual bool OnTouchEnd();
	void Reset();


	void LoadRoom(int nRoomId);
	int m_nCurrentRoom;

	// Managers
	EntityManager* m_pEntityManager;
	LevelManager* m_pLevelManager;
	GameStateManager* m_pGameStateManager;

	cmMesh& GetSphereMesh() { return m_sphere; }
	cmMesh& GetSceneAlphaMesh() { return m_sceneAlpha; }
	cmMesh& GetSceneMatteMesh() { return m_sceneMatte; }
	cmMesh& GetSceneSpecularMesh() { return m_sceneSpecular; }
	cmMesh& GetSceneDetailMesh() { return m_sceneDetail; }
	cmMesh& GetSceneCollisionMesh() { return m_sceneCollision; }
	std::wstring& GetSceneTexture(){ return m_sceneTexture; }

	cmMesh& GetDudeIdleMeshFace() { return m_dudeIdleMeshFace; }
	cmMesh& GetDudeIdleMeshPhong() { return m_dudeIdleMeshPhong; }
	cmMesh& GetDudeIdleMeshMatte() { return m_dudeIdleMeshMatte; }
	cmMesh& GetDudeIdleMeshSpecular() { return m_dudeIdleMeshSpecular; }

	cmMesh& GetDudeKickLMeshFace() { return m_dudeKickLMeshFace; }
	cmMesh& GetDudeKickLMeshPhong() { return m_dudeKickLMeshPhong; }
	cmMesh& GetDudeKickLMeshMatte() { return m_dudeKickLMeshMatte; }
	cmMesh& GetDudeKickLMeshSpecular() { return m_dudeKickLMeshSpecular; }

	cmMesh& GetDudeKickRMeshFace() { return m_dudeKickRMeshFace; }
	cmMesh& GetDudeKickRMeshPhong() { return m_dudeKickRMeshPhong; }
	cmMesh& GetDudeKickRMeshMatte() { return m_dudeKickRMeshMatte; }
	cmMesh& GetDudeKickRMeshSpecular() { return m_dudeKickRMeshSpecular; }

	cmMesh& GetDudePunchLMeshFace() { return m_dudePunchLMeshFace; }
	cmMesh& GetDudePunchLMeshPhong() { return m_dudePunchLMeshPhong; }
	cmMesh& GetDudePunchLMeshMatte() { return m_dudePunchLMeshMatte; }
	cmMesh& GetDudePunchLMeshSpecular() { return m_dudePunchLMeshSpecular; }

	cmMesh& GetDudePunchRMeshFace() { return m_dudePunchRMeshFace; }
	cmMesh& GetDudePunchRMeshPhong() { return m_dudePunchRMeshPhong; }
	cmMesh& GetDudePunchRMeshMatte() { return m_dudePunchRMeshMatte; }
	cmMesh& GetDudePunchRMeshSpecular() { return m_dudePunchRMeshSpecular; }

	cmMesh& GetFlameMesh() { return m_flameMesh; }

	Vector3& GetLightPos() { return m_vLightPos; }
	Camera& GetCamera() { return m_camera; }
	cmMeshCollider& GetCollision() { return m_collision; }
	EntityManager* GetEntityManager() { return m_pEntityManager; }
	GameStateManager* GetGameStateManager() { return m_pGameStateManager; }
	float GetZConstraint() const { return m_fZConstraint; }

	cmColor GetCurrentPlayerColour() { return GetPlayerColour(m_playerTurn); }
	cmColor GetPlayerColour(Player player);
	void SetPlayerTurn(Player turn) { m_playerTurn = turn; }
	Player m_playerTurn = Player::Blue;

private:
	void RenderBackground();
	void RenderScene();
	void RenderLight();

	cmFramebuffer m_BufferA;
	cmFramebuffer m_BufferB;
	cmMesh m_sceneAlpha;
	cmMesh m_sceneMatte;
	cmMesh m_sceneSpecular;
	cmMesh m_sceneDetail;
	cmMesh m_sceneCollision;
	cmMesh m_sphere;
	std::wstring m_sceneTexture;

	cmMesh m_dudeIdleMeshFace;
	cmMesh m_dudeIdleMeshPhong;
	cmMesh m_dudeIdleMeshMatte;
	cmMesh m_dudeIdleMeshSpecular;

	cmMesh m_dudeKickLMeshFace;
	cmMesh m_dudeKickLMeshPhong;
	cmMesh m_dudeKickLMeshMatte;
	cmMesh m_dudeKickLMeshSpecular;

	cmMesh m_dudeKickRMeshFace;
	cmMesh m_dudeKickRMeshPhong;
	cmMesh m_dudeKickRMeshMatte;
	cmMesh m_dudeKickRMeshSpecular;

	cmMesh m_dudePunchLMeshFace;
	cmMesh m_dudePunchLMeshPhong;
	cmMesh m_dudePunchLMeshMatte;
	cmMesh m_dudePunchLMeshSpecular;

	cmMesh m_dudePunchRMeshFace;
	cmMesh m_dudePunchRMeshPhong;
	cmMesh m_dudePunchRMeshMatte;
	cmMesh m_dudePunchRMeshSpecular;

	cmMesh m_flameMesh;

	cmMeshCollider m_collision;
	Camera m_camera;
	Vector3 m_vLightPos;
	float m_fZConstraint;
	
};