#pragma once

#include "GameState.h"
#include <typeinfo>
#include <type_traits>

class GameStateManager
{
public:
	GameStateManager();
	virtual ~GameStateManager();

	void Update();
	void Render();

	template<typename State, typename ...Args> 
	State* PushGameState(Args...);
	GameState* GetCurrentState() { return m_pCurrentState; }

	bool OnMouseDown(const cmMouseButtons& button, const Vector3& vPos);
	bool OnMouseUp(const cmMouseButtons& button, const Vector3& vPos);
	bool OnMouseClick(const cmMouseButtons& button, const Vector3& vPos);
	bool OnMouseMove(const Vector3& vMousePos);
	bool OnTouchStart();
	bool OnTouchEnd();

protected:
	GameState* m_pCurrentState = nullptr;
};

template<class State, typename... Args>
State* GameStateManager::PushGameState(Args... args)
{
	if (m_pCurrentState)
	{
		if (typeid(m_pCurrentState) == typeid(State))
			return (State*)m_pCurrentState;
		
		m_pCurrentState->OnEnd();
		delete m_pCurrentState;
		m_pCurrentState = nullptr;
	}

	m_pCurrentState = new State(args...);
	m_pCurrentState->OnStart();

	return (State*)m_pCurrentState;
}


