#include "MainMenuScreen.h"
#include "InGameScreen.h"
#include "cmGUI.h"
#include "GUIAnimations.h"
#include "cmPlatform.h"
#include "GGJ2015.h"
#include "PlayState.h"
#include "CreditsScreen.h"
#include "TutorialScreen.h"


const wchar_t* MainMenuScreen::ms_szScreenName = L"MainMenuScreen";
const int MainMenuScreen::ms_nAnimFrames = 25;
const float MainMenuScreen::ms_fAnimTime = 1000.0f;
MainMenuScreen::MainMenuScreen()
: m_pImgLogo(nullptr)
, m_pBtnPlay(nullptr)
{}

MainMenuScreen::~MainMenuScreen()
{}

void MainMenuScreen::OnInit()
{

	m_pImgLogo = new cmGUIImage();
	if (m_pImgLogo)
	{
		m_pImgLogo->SetPosition(Vector3(0.0f, 0.0f));
		m_pImgLogo->SetSize(Vector3(1.0f, 1.0f));
		m_pImgLogo->SetTexture(L"logo00000.tga");
		m_pImgLogo->SetAnchorParent(AnchorTop);
		m_pImgLogo->SetAnchorSelf(AnchorTop);
		m_pImgLogo->SetScaleMode(ModeScaledRelativeToHeightAspect);
		LOGO_FADE_OUT(m_pImgLogo, 0.0f, 300.0f, 300.0f);
		AddComponent(m_pImgLogo);
	}


	m_pBtnPlay = new cmGUIButton();
	if (m_pBtnPlay)
	{
		m_pBtnPlay->SetPosition(Vector3(0.0f, 0.35f));
		m_pBtnPlay->SetSize(Vector3(1.0f, 0.15f));
		m_pBtnPlay->SetFontSize(0.3f);
		m_pBtnPlay->SetTextOffset(Vector3(0.0f, -0.2f));
		m_pBtnPlay->SetText(L"STR_MAINMENU_BATTLE");
		m_pBtnPlay->SetColor(cmColor(1.0f, 1.0f, 1.0f, 1.0f));
		m_pBtnPlay->SetTextureOff(L"btn_menu_battle_idle.tga");
		m_pBtnPlay->SetAnchorParent(AnchorCenter);
		m_pBtnPlay->SetAnchorSelf(AnchorCenter);
		m_pBtnPlay->SetScaleMode(ModeScaledRelativeToHeightAspect);
		m_pBtnPlay->SetTouchPadding(Vector3(1.0f, 0.015f));
		BOTTOM_BUTTON_ANIM(m_pBtnPlay, 900.0f, 0.0f, 300.0f);
		AddComponent(m_pBtnPlay);
	}

	m_pBtnTutorial = new cmGUIButton();
	if (m_pBtnTutorial)
	{
		m_pBtnTutorial->SetPosition(Vector3(-0.35f, 0.35f));
		m_pBtnTutorial->SetSize(Vector3(1.0f, 0.125f));
		m_pBtnTutorial->SetFontSize(0.3f);
		m_pBtnTutorial->SetTextOffset(Vector3(0.0f, -0.025f));
		m_pBtnTutorial->SetText(L"STR_MAINMENU_TUTORIAL");
		m_pBtnTutorial->SetColor(cmColor(1.0f, 1.0f, 1.0f, 1.0f));
		m_pBtnTutorial->SetTextureOff(L"btn_menu_big_idle.tga");
		m_pBtnTutorial->SetAnchorParent(AnchorCenter);
		m_pBtnTutorial->SetAnchorSelf(AnchorCenter);
		m_pBtnTutorial->SetScaleMode(ModeScaledRelativeToHeightAspect);
		m_pBtnTutorial->SetTouchPadding(Vector3(1.0f, 0.015f));
		BOTTOM_BUTTON_ANIM(m_pBtnTutorial, 1000.0f, 100.0f, 300.0f);
		AddComponent(m_pBtnTutorial);
	}

	m_pBtnCredits = new cmGUIButton();
	if (m_pBtnCredits)
	{
		m_pBtnCredits->SetPosition(Vector3(0.35f, 0.35f));
		m_pBtnCredits->SetSize(Vector3(1.0f, 0.125f));
		m_pBtnCredits->SetFontSize(0.3f);
		m_pBtnCredits->SetTextOffset(Vector3(0.0f, -0.025f));
		m_pBtnCredits->SetText(L"STR_MAINMENU_CREDITS");
		m_pBtnCredits->SetColor(cmColor(1.0f, 1.0f, 1.0f, 1.0f));
		m_pBtnCredits->SetTextureOff(L"btn_menu_big_idle.tga");
		m_pBtnCredits->SetAnchorParent(AnchorCenter);
		m_pBtnCredits->SetAnchorSelf(AnchorCenter);
		m_pBtnCredits->SetScaleMode(ModeScaledRelativeToHeightAspect);
		m_pBtnCredits->SetTouchPadding(Vector3(1.0f, 0.015f));
		BOTTOM_BUTTON_ANIM(m_pBtnCredits, 1000.0f, 100.0f, 300.0f);
		AddComponent(m_pBtnCredits);
	}
}

void MainMenuScreen::OnEvent(GUIEvents eEvent, cmGUIComponent* pComponent)
{
	if (eEvent == EventClick)
	{
		if (pComponent == m_pBtnPlay)
		{
			TransitionTo(InGameScreen::ms_szScreenName, 0, true);
			bGoToGame = true;
		}
		if (pComponent == m_pBtnCredits)
		{
			TransitionTo(CreditsScreen::ms_szScreenName, 0, false);
		}
		if (pComponent == m_pBtnTutorial)
		{
			TransitionTo(TutorialScreen::ms_szScreenName, 0, false);
		}
	}
}

void MainMenuScreen::OnUpdate()
{

	if (m_fAnimTimer < ms_fAnimTime)
	{
		m_fAnimTimer += gPlatform->GetDeltaTime();
	}
	else
	{
		m_fAnimTimer = ms_fAnimTime;
	}

	if (m_pImgLogo)
	{
		wchar_t buffer[64];
		int nIndex = (int)(m_fAnimTimer / ms_fAnimTime * (float)ms_nAnimFrames);
		swprintf(buffer, sizeof(buffer) / sizeof(wchar_t), L"logo%05d.tga", nIndex);
		m_pImgLogo->SetTexture(buffer);

		if (nIndex == 0)
		{
			gAudio->Play(L"Space Punch title voice fx.ogg", 0.8f);
		}
		if (nIndex == 10)
		{
			gAudio->Play(L"Hit 1.ogg", 0.8f, 0.6f);
		}
		if (nIndex == 14)
		{
			gAudio->Play(L"blammo3.ogg", 1.0f);
		}
	}
}

static ALuint s_MusicHandleId = 0;
void MainMenuScreen::OnEnter()
{
	bGoToGame = false;
	m_fAnimTimer = 0;
	wchar_t buffer[64];
	for (int i = 0; i <= ms_nAnimFrames; ++i)
	{
		swprintf(buffer, sizeof(buffer) / sizeof(wchar_t), L"logo%05d.tga", i);
		gRenderer->GetTexture(buffer);
	}

	s_MusicHandleId = gAudio->PlayMusicLooping(L"Quirky BG.ogg", 0.4f);
}

void MainMenuScreen::OnExit()
{
	if (bGoToGame)
	{
		wchar_t buffer[64];
		for (int i = 0; i <= ms_nAnimFrames; ++i)
		{
			swprintf(buffer, sizeof(buffer) / sizeof(wchar_t), L"logo%05d.tga", i);
			gRenderer->ReloadTexture(buffer);
		}

		GGJ2015::Get()->GetGameStateManager()->PushGameState<PlayState>();

		gAudio->GetSource(s_MusicHandleId).Fade(0.0f, 3.0f);
	}
}