#pragma once
#include "GameState.h"
#include "Spaceman.h"
#include <memory>
#include "Player.h"
#include "Stages.h"

class BlastDoor;
class SpawnChute;

class PlayState : public GameState
{
	static const float ms_fMinimumScreenDragDistance;
	static const float ms_fMaximumScreenDragDistance;
	static const float ms_fVacuumAcceleration;
	static const float ms_fMaximumVacuumVelocity;
	static const float ms_fVacuumTimeMS;
	static const float ms_fWinningTimeMs;
	static const int ms_nMaxPunches;
	static const int ms_nMaxSpacemen;
	
public:

	enum State
	{
		Respawn,
		Select,			// User is selecting a spaceman to use
		PrepareLaunch, // User has a spaceman selected and can launch him
		Launched,		// Spaceman is in flight and capable of interacting with the world
		TransitionStage_Start,// a player has won the round and we are progressing to the next level
		TransitionStage_Vaccuum, 
		TransitionStage_MoveStage,
		TransitionStage_BuildLevel,
		TransitionStage_Exhale,
		
		Winning,
	};



	PlayState();
	virtual ~PlayState();

	virtual void OnStart();
	virtual void Update();
	virtual void Render();
	virtual void OnEnd() {}

	virtual bool OnMouseDown(const cmMouseButtons& button, const Vector3& vPos);
	virtual bool OnMouseMove(const Vector3& vMousePos);
	virtual bool OnMouseUp(const cmMouseButtons& button, const Vector3& vPos);
	bool GetScreenSpaceMouseWorldIntersect(Vector3& vPos);
	virtual bool OnTouchStart();
	virtual bool OnTouchEnd();
	bool bDragging = false;

	Vector3 m_touchPosition;

	float CalculateLaunchPower(const Vector3& releasePosition);

	void SetSpaceman(Spaceman* pSpaceman);
	void ClearSelectedSpaceman();
	// TODO: weakref? what if it dies in transit?
	Spaceman* m_pSelectedSpaceman = nullptr;
	int m_nNumPunches = 0;

	// CurrentState
	void SetState(State state) { m_currentPlayState = state; m_fStageTimer = 0.0f; }
	State m_currentPlayState;

	// Player turn handover
	void EndTurn();
	void ChangePlayer();

	BlastDoor* m_pRedDoor = nullptr;
	BlastDoor* m_pBlueDoor = nullptr;

	SpawnChute* m_pRedSpawnChute = nullptr;
	SpawnChute* m_pBlueSpawnChute = nullptr;

	void RespawnSpaceman(Player owner);

	// Stage handover
	
	void RebuildLevel(bool bInitial = false);
	void SpawnHazard(cmTransform& transform, const std::string& sTrigger);
	float m_fStageTimer = 0.0f;
	void SetPlayerStageWinner(Player winner);
	RNG m_hazardRNG;
	Player m_stageWinner = Player::None;
	Stage m_currentStage = Stage::Stage_Start;
	void SuckSpacemenTowardStageDoor(Player player = Player::None);
	void SuckSpacemenToward(const Vector3& position, Player player = Player::None);
	unsigned int m_nSuctionSample = 0;
};

