#include "Spaceman.h"
#include "cmPlatform.h"
#include "GGJ2015.h"
#include "NameGenerator.h"
#include <string>

//======================================================================
const float Spaceman::ms_passiveRotationSpeed = 0.01f;
const float Spaceman::ms_bounceRotationSpeed = 25.5f;
const float Spaceman::ms_rotationDecay = 0.15f;
const float Spaceman::ms_sphereRadius = 3.75f;
const float Spaceman::ms_punchRadius = 7.0f;
const float Spaceman::ms_punchPower = 0.55f;
const float Spaceman::ms_bouncinessWall = 0.9f; //0.9f
const float Spaceman::ms_bouncinessSpacemen = 0.9f; //0.9f
const float Spaceman::ms_airFriction = 0.25f; //0.25f
const float Spaceman::ms_maximumLaunchVelocity = 0.03f;
const float Spaceman::ms_fHeightSmoothFactor = 0.2f;
const float Spaceman::ms_fPunchTime = 0.25f;
//======================================================================

Spaceman::Spaceman(Player player, float x, float y) : Entity()
	, m_player(player)
{
	m_bIsColliding = false;
	m_transform.position = Vector3(x, y, GGJ2015::Get()->GetZConstraint());
	m_fullname = NameGenerator::GetRandomName();
	m_tidbit = NameGenerator::GetRandomTidbit();
	//m_velocity = Vector3(Vector3::right() * ((float)m_rng.Get() / (float)RNG::s_nMax) * 0.05f + Vector3::forward() * ((float)m_rng.Get() / (float)RNG::s_nMax) * 0.05f);
	
	float rotation = (FTAU * ((float)m_rng.Get() / (float)RNG::s_nMax));
	m_transform.yaw(rotation);
	m_fRotationAmount = ms_passiveRotationSpeed * (m_rng.Get()%2 == 0 ? -1.0f : 1.0f);
	
	m_collision = new SphereCollision(this, ms_sphereRadius);
	m_velocity = Vector3();
	m_ePunchDirection = ePunchDirection::None;
	m_fPunchTimer = 0.0f;
}

Spaceman::~Spaceman()
{}

void Spaceman::PreUpdate()
{
	m_resultantVelocity = m_velocity;
	m_resultantPosition = m_transform.position;
}

void Spaceman::Update()
{
	float fDeltaSeconds = (gPlatform->GetDeltaTime() / 1000.0f);

	if (m_fNoClipTimer > 0.0f)
	{
		m_fNoClipTimer -= gPlatform->GetDeltaTime();
	}
	if (m_collision)
	{
		m_collision->bEnabled = m_fNoClipTimer <= 0.0f;
	}

	if (m_fPunchTimer > 0.0f)
	{
		m_fPunchTimer -= fDeltaSeconds;
		if (m_fPunchTimer <= 0.0f)
		{
			m_fPunchTimer = 0.0f;
			m_ePunchDirection = ePunchDirection::None;
		}
	}

	//apply collision velocity and position
	m_velocity = m_resultantVelocity;
	m_transform.position = m_resultantPosition;

	// Rotate him
	float rotation = (FTAU * m_fRotationAmount * fDeltaSeconds);
	m_transform.yaw(rotation);

	m_transform.translateWorld(m_velocity * gPlatform->GetDeltaTime());

	Vector3 vCollisionPoint, vNormal;
	if (m_collision->bEnabled && GGJ2015::Get()->GetCollision().SphereTest(m_transform.position, ms_sphereRadius, vCollisionPoint, vNormal))
	{
		//reflect by collision normal
		vNormal = vNormal.normalize();
		m_velocity = m_velocity - vNormal * 2.0f * m_velocity.dot(vNormal);

		if (!m_bIsColliding)
		{
			gAudio->Play(L"Wall impact 1.ogg", 0.8f, 0.6f);
		}

		{
			m_velocity = m_velocity * ms_bouncinessWall;

			m_fRotationAmount = (m_rng.Get() % 100 > 50 ? ms_bounceRotationSpeed * 1.0f : ms_bounceRotationSpeed * -1.0f) * m_velocity.magnitude();
			m_bIsColliding = true;

			//move away from wall
			Vector3 vCollision = vCollisionPoint - m_transform.position;
			float fSeparationDist = (ms_sphereRadius - vCollision.magnitude()) + 0.0001f;
			m_transform.position = m_transform.position - vCollision.normalize() * fSeparationDist;
		}
	}
	else
	{
		m_bIsColliding = false;
	}

	float heightDiff = GGJ2015::Get()->GetZConstraint() - m_transform.position.z;
	float zFactor = ms_fHeightSmoothFactor * fDeltaSeconds;
	m_transform.position.z += heightDiff * zFactor;
	

	std::vector<std::string> triggers = cmLocatorManager::Get().SphereTriggerTest(m_transform.position, ms_sphereRadius);
	for (int i = 0; i < (int)triggers.size(); ++i)
	{
		if (triggers[i].compare("fire_trigger_001") == 0)
		{
		}
	}

	m_velocity.z = 0;
	m_velocity -= m_velocity * ms_airFriction * fDeltaSeconds;

	if (fabs(m_fRotationAmount) < ms_passiveRotationSpeed)
	{
		m_fRotationAmount = m_fRotationAmount / fabs(m_fRotationAmount) * ms_passiveRotationSpeed;
	}
	else
	{
		m_fRotationAmount -= m_fRotationAmount * ms_rotationDecay * fDeltaSeconds;
	}
	
}

void Spaceman::Render()
{
	cmMesh* meshFace = nullptr;
	cmMesh* meshMatte = nullptr;
	cmMesh* meshPhong = nullptr;
	cmMesh* meshSpecular = nullptr;
	std::wstring strFaceTex;
	switch (m_ePunchDirection)
	{
		case BottomLeft:
			meshFace = &GGJ2015::Get()->GetDudeKickLMeshFace();
			meshMatte = &GGJ2015::Get()->GetDudeKickLMeshMatte();
			meshPhong = &GGJ2015::Get()->GetDudeKickLMeshPhong();
			meshSpecular = &GGJ2015::Get()->GetDudeKickLMeshSpecular();
			strFaceTex = L"spaceman_01_head_attackdown.tga";
			break;
		case BottomRight:
			meshFace = &GGJ2015::Get()->GetDudeKickRMeshFace();
			meshMatte = &GGJ2015::Get()->GetDudeKickRMeshMatte();
			meshPhong = &GGJ2015::Get()->GetDudeKickRMeshPhong();
			meshSpecular = &GGJ2015::Get()->GetDudeKickRMeshSpecular();
			strFaceTex = L"spaceman_01_head_attackdown.tga";
			break;
		case TopLeft:
			meshFace = &GGJ2015::Get()->GetDudePunchLMeshFace();
			meshMatte = &GGJ2015::Get()->GetDudePunchLMeshMatte();
			meshPhong = &GGJ2015::Get()->GetDudePunchLMeshPhong();
			meshSpecular = &GGJ2015::Get()->GetDudePunchLMeshSpecular();
			strFaceTex = L"spaceman_01_head_attackup.tga";
			break;
		case TopRight:
			meshFace = &GGJ2015::Get()->GetDudePunchRMeshFace();
			meshMatte = &GGJ2015::Get()->GetDudePunchRMeshMatte();
			meshPhong = &GGJ2015::Get()->GetDudePunchRMeshPhong();
			meshSpecular = &GGJ2015::Get()->GetDudePunchRMeshSpecular();
			strFaceTex = L"spaceman_01_head_attackup.tga";
			break;
		default:
			meshFace = &GGJ2015::Get()->GetDudeIdleMeshFace();
			meshMatte = &GGJ2015::Get()->GetDudeIdleMeshMatte();
			meshPhong = &GGJ2015::Get()->GetDudeIdleMeshPhong();
			meshSpecular = &GGJ2015::Get()->GetDudeIdleMeshSpecular();
			strFaceTex = L"spaceman_01_head_idle.tga";
			break;
	}

	if (meshFace && meshMatte && meshPhong && meshSpecular)
	{
		gRenderer->MatrixPush();
		gRenderer->SetProjectionMode(Mode3D);
		gRenderer->MatrixMultiply(m_transform.matrix());

		gRenderer->MatrixPush();
		gRenderer->MatrixRotate(-90, Vector3(1.0f, 0.0f, 0.0f));
		gRenderer->MatrixPush();
		float fBig = 1.5f;
		gRenderer->MatrixScale(Vector3(fBig, fBig, fBig));
		cmFlatShader::Get().Activate();
		gRenderer->SetColor(cmColor(1.0f, 1.0f, 1.0f, 1.0f));
		gRenderer->BindTexture(gRenderer->GetTexture(strFaceTex).nId);


		meshFace->Render();

		if (m_player == Player::Blue)
		{
			gRenderer->BindTexture(gRenderer->GetTexture(L"spaceman_01_body.tga").nId);
		}
		else
		{
			gRenderer->BindTexture(gRenderer->GetTexture(L"spaceman_01_body_red.tga").nId);
		}

		meshMatte->Render();
		meshPhong->Render();
// 		gRenderer->UpdateModelviewMatrix();
// 		gRenderer->UpdateProjectionMatrix();
// 		cmAlphaSpecShader::Get().Activate();
// 		cmAlphaSpecShader::Get().SetLightPos(GGJ2015::Get()->GetLightPos() * GGJ2015::Get()->GetCamera().GetTransform().viewMatrix());
// 		cmAlphaSpecShader::Get().SetSpecularColor(cmColor());
// 		cmAlphaSpecShader::Get().SetSpecularGloss(45.0f);
// 		cmAlphaSpecShader::Get().SetSpecularAmount(0.2f);
// 		cmAlphaSpecShader::Get().SetModelViewMatrix(gRenderer->GetModelViewMatrix());
// 		cmAlphaSpecShader::Get().SetNormalMatrix(gRenderer->GetNormalMatrix());
// 		meshSpecular->Render();
// 		gRenderer->MatrixPop();
// 		gRenderer->MatrixPop();

		// 	gRenderer->MatrixPush();
		gRenderer->UpdateModelviewMatrix();
		gRenderer->UpdateProjectionMatrix();
		// 	cmPhongShader::Get().Activate();
		// 	cmPhongShader::Get().SetAmbient(0.25f);
		// 	cmPhongShader::Get().SetLightPos(GGJ2015::Get()->GetLightPos() * GGJ2015::Get()->GetCamera().GetTransform().viewMatrix());
		// 	cmPhongShader::Get().SetSpecularColor(cmColor());
		// 	cmPhongShader::Get().SetSpecularGloss(30.0f);
		// 	cmPhongShader::Get().SetSpecularAmount(0.5f);
		// 	cmPhongShader::Get().SetModelViewMatrix(gRenderer->GetModelViewMatrix());
		// 	cmPhongShader::Get().SetNormalMatrix(gRenderer->GetNormalMatrix());
		// 
		// 	if (m_bIsColliding)
		// 		gRenderer->SetColor(cmColor(0.4f, 1.0f, 0.4f, 1.0f));
		// 	else
		// 		gRenderer->SetColor(GGJ2015::Get()->GetPlayerColour(m_player));
		// 
		// 	gRenderer->BindTexture(gRenderer->GetTexture(L"TEX_WHITE").nId);
		// 	((GGJ2015*)gGame)->GetDudeMesh().Render();
		// 	gRenderer->MatrixPop();
		// 
		gRenderer->BindTexture(gRenderer->GetTexture(L"TEX_WHITE").nId);
		gRenderer->MatrixPush();
		gRenderer->UpdateModelviewMatrix();
		gRenderer->UpdateProjectionMatrix();
		gRenderer->MatrixScale(Vector3(ms_sphereRadius, ms_sphereRadius, ms_sphereRadius));
		gRenderer->UpdateModelviewMatrix();
		gRenderer->UpdateProjectionMatrix();
		cmPhongShader::Get().SetModelViewMatrix(gRenderer->GetModelViewMatrix());
		cmPhongShader::Get().SetNormalMatrix(gRenderer->GetNormalMatrix());
		gRenderer->SetColor(cmColor(0.75f, 0.75f, 0.75f, 0.25f));
		//((GGJ2015*)gGame)->GetSphereMesh().Render();
		gRenderer->MatrixPop();

		gRenderer->MatrixPop();
	}
}

void Spaceman::OnCollision(Entity* pEntity) 
{
	Spaceman* pOtherSpaceman = dynamic_cast<Spaceman*>(pEntity);
	if (pOtherSpaceman)
	{
		//I hit another spaceman
		ElasticSphereCollision(m_transform.position, m_velocity, ((SphereCollision*)m_collision)->m_fRadius, 1.0f, m_resultantPosition, m_resultantVelocity, pOtherSpaceman->m_transform.position, pOtherSpaceman->m_velocity, ((SphereCollision*)pOtherSpaceman->m_collision)->m_fRadius, 1.0f, pOtherSpaceman->m_resultantPosition, pOtherSpaceman->m_resultantVelocity);
		m_velocity = m_velocity * ms_bouncinessSpacemen;
		if (!m_bIsColliding)
		{
			gAudio->Play(L"Wall impact 1.ogg", 0.9f, 0.6f);
		}
	}
}

void Spaceman::Launch(const Vector3& direction, float fPower)
{
	fPower = clampFloat(fPower, 0.0f, 1.0f);
	m_velocity += direction * (fPower * ms_maximumLaunchVelocity);
}

void Spaceman::Punch(ePunchDirection dir)
{
	m_ePunchDirection = dir;
	m_fPunchTimer = ms_fPunchTime;
	
	int punchId = 1 + (m_rng.Get() % 7);
	std::wstring punchSample(L"Punch Noise " + toStringW(punchId) + L".ogg");
	gAudio->Play(punchSample, 0.7f);
}

//======================================================================